unit DLog;

interface

uses Classes;

const
  {$I CompConstants.inc}

  cTimeStampCol         = 1; // The Index of the TimeStamp Column
  cExportTitle          = 'Debug Log';

type
  TDebugEvent = procedure (Sender: TObject; Choice: Integer) of object;

  TDebugLog = class(TComponent)
  private
    FLogging,            // Is Logging Enabled?
    FVisible,            // Log Visible on Startup?
    FTimestamp,          // Time Stamp each entry?
    FMaximized,          // Maximized on Startup
    FUpdating: Boolean;  // Has BeginUpdate has been called?

    FCurrencyDigits: Integer; // Number of Digits to display for the Currency Type

    FChoices: TStrings;

    FOnDebug: TDebugEvent;

    procedure SetLogging(Value: Boolean);
    procedure SetVisible(Value: Boolean);
    procedure SetMaximized(Value: Boolean);
    procedure SetTimeStamp(Value: Boolean);
    procedure SetCurrencyDigits(Value: Integer);
    procedure SetChoices(Value: TStrings);

    function BoolToStr(Val: Boolean): String;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Loaded; override;

    procedure Add(EntryType: String; Information: array of Variant);
    procedure AddStrings(EntryType: String; S: TStrings);

    procedure BeginUpdate;
    procedure EndUpdate;

    procedure Hide;
    procedure Show;

    procedure Clear;

    procedure AddChoice(Choice: String);

  published
    // *** Properties ***
    property Logging: Boolean read FLogging write SetLogging default True;
    property Visible: Boolean read FVisible write SetVisible default True;
    property TimeStamp: Boolean read FTimeStamp write SetTimeStamp default True;
    property Maximized: Boolean read FMaximized write SetMaximized default False;

    property CurrencyDigits: Integer read FCurrencyDigits write SetCurrencyDigits default 2;
    property Choices: TStrings read FChoices write SetChoices;

    // *** Events ***
    property OnDebug: TDebugEvent read FOnDebug write FOnDebug;
  end;

implementation

uses Forms, Controls, SysUtils, ShellApi, Windows, ComCtrls, Dialogs,

     // DebugLog Units
     DebugLogForm;

// *** TDebugLog Methods ***

constructor TDebugLog.Create(AOwner: TComponent);
begin
  inherited;

  // Set Logging Property
  FLogging := True; // Default to Logging

  // Initialize Variables
  FUpdating := False;
  FChoices := TStringList.Create;

  // Create Debug Window
  DebugForm := TDebugForm.Create(nil);
  DebugForm.LogComponent := Self;

  // Set Other Property Defaults
  FVisible   := True;
  FMaximized := False;
  FTimeStamp := True;

  FCurrencyDigits := 2;
end;

procedure TDebugLog.Loaded;
begin
  inherited;

  SetVisible(FVisible);
  SetMaximized(FMaximized);
  SetTimeStamp(FTimeStamp);
end;

destructor TDebugLog.Destroy;
begin
  DebugForm.Free;

  inherited;
end;

procedure TDebugLog.SetLogging(Value: Boolean);
const
  cLogging    = 'Logging';
  cNotLogging = 'Not Logging';
begin
  if Value <> FLogging  then
  with DebugForm do
  begin
    ChoicesBox.Enabled := Value;
    if Value then
      StatusBar.Panels[0].Text := cLogging
    else StatusBar.Panels[0].Text := cNotLogging;
    FLogging := Value;
  end;
end;

procedure TDebugLog.SetVisible(Value: Boolean);
begin
  if not (csDesigning in ComponentState) then
    DebugForm.Visible := Value;

  FVisible := Value;
end;

procedure TDebugLog.SetMaximized(Value: Boolean);
begin
  if FLogging and (Value <> FMaximized) then
  begin
    if not (csDesigning in ComponentState) then
      with DebugForm do
        if Value then
          WindowState := wsMaximized
        else WindowState := wsNormal;

    FMaximized := Value;
  end;
end;

procedure TDebugLog.SetTimeStamp(Value: Boolean);
begin
  if FLogging and (Value <> FTimeStamp) then
  begin
    if not (csDesigning in ComponentState) then
      with DebugForm.DebugList do
        if Columns.Count > cTimeStampCol then
          with Columns[cTimeStampCol] do
            if Value then
              Width := -1
            else Width := 0;

    FTimeStamp := Value;
  end;
end;

procedure TDebugLog.SetCurrencyDigits(Value: Integer);
begin
  if (Value <> FCurrencyDigits) and (Value in [0..4]) then
    FCurrencyDigits := Value;
end;

procedure TDebugLog.SetChoices(Value: TStrings);
begin
  FChoices.Assign(Value);

  if not (csDesigning in ComponentState) then
    DebugForm.ChoicesBox.Items.Assign(Value); // Update the Choices Box
end;

function TDebugLog.BoolToStr(Val: Boolean): String;
begin
  if Val then
    Result := 'True'
  else Result := 'False';
end;

procedure TDebugLog.Add(EntryType: String; Information: array of Variant);
var
  NewItem: TListItem;
  S: String;
  i: Integer;
begin
  if FLogging then
  begin
    // Add the Log Entry
    NewItem := DebugForm.DebugList.Items.Add;
    if Assigned(NewItem) then
      with DebugForm.DebugList, NewItem do
      begin
        // If this is the first item added, enable Toolbar
        if Items.Count = 1 then
          DebugForm.EnableToolbar(True);

        // Add the Item No, TimeStamp and Log Entry Type
        Caption := IntToStr(Items.Count);
        SubItems.Add(FormatDateTime('hh:nn:ss', Time)); // Log Time
        SubItems.Add(EntryType);       // Log Entry Type

        // Add the Information                                                  currency
        for i := Low(Information) to High(Information) do
        begin
          // Get Information to String Format so it can be displayed in ListView
          case VarType(Information[i]) and varTypeMask of
            varInteger, varSmallInt, varByte: S := IntToStr(Information[i]);
            varSingle, varDouble            : S := FloatToStr(Information[i]);
            varCurrency                     : S := FloatToStrF(Information[i], ffCurrency, 14, FCurrencyDigits);
            varDate                         : S := DateTimeToStr(Information[i]);
            varBoolean                      : S := BoolToStr(Information[i]);
            varString                       : S := Information[i];
          end;

          // Add to Debug Log
          SubItems.Add(S);
        end;

        // Add more columns if necessary
        if Columns.Count < SubItems.Count + 1 then
          repeat
            with Columns.Add do
            begin
              Width := -1; // Resize Column to Item Width
              Caption := 'Value ' + IntToStr(Index - 2);
            end
          until Columns.Count = SubItems.Count + 1;

        // Make the Item Visible;
        if not FUpdating then
          MakeVisible(False);
      end;
    end;
end;

procedure TDebugLog.AddStrings(EntryType: String; S: TStrings);
const
  cMinNeededToStartUpdate = 20;
var
  NewItem: TListItem;
  i: Integer;
begin
  if FLogging then
  begin
    // Add the Log Entry
    NewItem := DebugForm.DebugList.Items.Add;
    if Assigned(NewItem) then
      with DebugForm.DebugList, NewItem do
      try
        if S.Count >= cMinNeededToStartUpdate then
          Items.BeginUpdate;

        // If this is the first item added, enable Toolbar
        if Items.Count = 1 then
          DebugForm.EnableToolbar(True);

        // Add the Item No, TimeStamp and Log Entry Type
        Caption := IntToStr(Items.Count);
        SubItems.Add(FormatDateTime('hh:nn:ss', Time)); // Log Time
        SubItems.Add(EntryType);       // Log Entry Type

        // Add the Information
        for i := 0 to S.Count - 1 do
          SubItems.Add(S[i]);

        // Add more columns if necessary
        if Columns.Count < SubItems.Count + 1 then
          repeat
            with Columns.Add do
            begin
              Width := -1; // Resize Column to Item Width
              Caption := 'Value ' + IntToStr(Index - 2);
            end
          until Columns.Count = SubItems.Count + 1;

        // Make the Item Visible;
        if not FUpdating then
          MakeVisible(False);

      finally
        if S.Count >= cMinNeededToStartUpdate then
          Items.EndUpdate;
      end;
    end;
end;

procedure TDebugLog.BeginUpdate;
begin
  if FLogging then
  begin
    FUpdating := True; // Flag Updating
    Screen.Cursor := crHourglass;
    DebugForm.DebugList.Items.BeginUpdate;
  end;
end;

procedure TDebugLog.EndUpdate;
begin
  with DebugForm.DebugList do
  try
    Items.EndUpdate;
  finally
    Screen.Cursor := crDefault;
    FUpdating := False; // Unflag Updating
    Items[Items.Count - 1].MakeVisible(False); // Make Last Item Visible
  end;
end;

procedure TDebugLog.Hide;
begin
  SetVisible(False);
end;

procedure TDebugLog.Show;
begin
  SetVisible(True);
end;

procedure TDebugLog.Clear;
begin
  DebugForm.Clear;
end;

procedure TDebugLog.AddChoice(Choice: String);
begin
  FChoices.Add(Choice); // Add the Choice
  DebugForm.ChoicesBox.Items.Assign(FChoices); // Update the Choices Box
end;

end.
